@@ -5,7 +5,7 @@ import subprocess
 import sys
 import tempfile
 from pathlib import Path, PurePath
-from typing import Union
+from typing import Union, List
 from urllib import request
 
 import requests
@@ -13,40 +13,21 @@ import typer
 
 from .extend_js import extend_js
 from .file_patterns import PkgFileFilter
-from .filter_env import filter_env, filter_pkg, iterate_env_pkg_meta
+from .filter_env import filter_env, iterate_env_pkg_meta, split_filter_pkg
 
 EMSDK_VER = "latest"
 
 HERE = Path(__file__).parent
 
 
-def shrink_conda_meta(prefix, target_dir=None):
-    if not target_dir:
-        target_dir = Path(prefix)
-
-    prefix = Path(prefix)
-
-    conda_meta = prefix / "conda-meta"
-    target_dir = Path(target_dir) / "conda-meta"
-    target_dir.mkdir(parents=True, exist_ok=True)
-
-    for p in conda_meta.glob("*.json"):
-        with open(p, "r") as file:
-            data = json.load(file)
-
-            data.pop("paths_data", None)
-            data.pop("files", None)
-
-        with open(target_dir / p.name, "w") as outfile:
-            json.dump(data, outfile)
-
-
 def download_and_setup_emsdk(emsdk_version):
     tag = None
     if emsdk_version == "latest":
         tags = requests.get("https://api.github.com/repos/emscripten-core/emsdk/tags")
-        if tags.content:
+        if tags.ok and tags.content:
             tag = json.loads(tags.content)[0]["name"]
+        else:
+            tag = "3.1.2"
     else:
         tag = emsdk_version
 
@@ -87,7 +68,8 @@ def get_file_packager_path():
     try:
         import emsdk
 
-        emsdk_dir = Path(emsdk.__file__).parent
+        if emsdk.__file__ is not None:
+            emsdk_dir = Path(emsdk.__file__).parent
     except ImportError:
         pass
 
@@ -180,63 +162,187 @@ def split_pack_environment(
     pkg_file_filter: PkgFileFilter,
     pack_outdir: Union[str, None] = None,
     download_emsdk: Union[str, None] = None,
+    silent: bool = False,
 ):
     if pack_outdir is None:
         pack_outdir = os.getcwd()
 
     # name of the env
     env_name = PurePath(env_prefix).parts[-1]
-    print(env_name)
-
-    all_pks = []
+    js_files = []
     for pkg_meta in iterate_env_pkg_meta(env_prefix):
 
         pkg_outname = pkg_meta["fn"][:-8]
+        pkg_name = pkg_meta["name"]
 
-        with tempfile.TemporaryDirectory() as temp_dir:
-            target_dir = os.path.join(temp_dir, env_name)
-            os.makedirs(target_dir)
+        matchers = pkg_file_filter.get_matchers(pkg_name=pkg_name)
 
-        has_any_file = filter_pkg(
-            env_prefix=env_prefix,
-            pkg_meta=pkg_meta,
-            target_dir=target_dir,
-            pkg_file_filter=pkg_file_filter,
-        )
-        if has_any_file:
-            all_pks.append(pkg_outname)
-            emscripten_file_packager(
-                outname=pkg_outname,
-                to_mount=env_name,
-                mount_path=env_prefix,
-                export_name=export_name,
-                use_preload_plugins=True,
-                no_node=export_name.startswith("globalThis"),
-                lz4=True,
-                cwd=str(temp_dir),
-                download_emsdk=download_emsdk,
+        with tempfile.TemporaryDirectory() as temp_root_dir:
+
+            temp_dirs = []
+            target_dirs = []
+            for i in range(len(matchers)):
+                temp_dir = os.path.join(temp_root_dir, f"{i}")
+                target_dir = os.path.join(temp_dir, env_name)
+                os.makedirs(target_dir)
+                temp_dirs.append(temp_dir)
+                target_dirs.append(target_dir)
+
+            has_any_files = split_filter_pkg(
+                env_prefix=env_prefix,
+                pkg_meta=pkg_meta,
+                target_dirs=target_dirs,
+                matchers=matchers,
             )
 
-            shutil.copy(os.path.join(str(temp_dir), f"{pkg_outname}.data"), pack_outdir)
-            shutil.copy(os.path.join(str(temp_dir), f"{pkg_outname}.js"), pack_outdir)
+            for i in range(len(matchers)):
+                if has_any_files[i]:
+                    emscripten_file_packager(
+                        outname=f"{pkg_outname}.{i}",
+                        to_mount=env_name,
+                        mount_path=env_prefix,
+                        export_name=export_name,
+                        use_preload_plugins=True,
+                        no_node=export_name.startswith("globalThis"),
+                        lz4=True,
+                        cwd=str(temp_dirs[i]),
+                        download_emsdk=download_emsdk,
+                        silent=silent,
+                    )
+                    js_files.append(f"{pkg_outname}.{i}.js")
+                    shutil.copy(
+                        os.path.join(str(temp_dirs[i]), f"{pkg_outname}.{i}.data"),
+                        pack_outdir,
+                    )
+                    shutil.copy(
+                        os.path.join(str(temp_dirs[i]), f"{pkg_outname}.{i}.js"),
+                        pack_outdir,
+                    )
 
-            extend_js(os.path.join(pack_outdir, f"{pkg_outname}.js"))
+                    extend_js(os.path.join(pack_outdir, f"{pkg_outname}.{i}.js"))
 
     # create the base js file
     lines = []
-    for pkg_outname in all_pks:
-        lines.append(f"    promises.push(import('./{pkg_outname}.js'));")
+    for js_file in js_files:
+        lines.append(f"    promises.push(import('./{js_file}'));")
     txt = "\n".join(lines)
-    js_import_all_func = f"""export default async function(){{
+
+    if export_name.startswith("globalThis"):
+        js_import_all_func = f"""export default async function(){{
     let promises = [];
 {txt}
     await Promise.all(promises);
 }}
-    """
+        """
+    else:
+        js_import_all_func = f"""async function importPackages(){{
+    let promises = [];
+{txt}
+    await Promise.all(promises);
+}}
+module.exports = importPackages
+        """
+
     with open(os.path.join(pack_outdir, f"{outname}.js"), "w") as f:
         f.write(js_import_all_func)
 
 
+def pack_repodata(
+    repodata: dict,
+    env_prefix: Path,
+    channels: List,
+    export_name: str,
+    pkg_file_filter: PkgFileFilter,
+    pack_outdir: Union[str, None] = None,
+    download_emsdk: Union[str, None] = None,
+):
+
+    if pack_outdir is None:
+        pack_outdir = os.getcwd()
+
+    packages = repodata["packages"]
+    for pkg_key, pkg_meta in packages.items():
+
+        # for each pkg we need to make sure env prefix is empty
+        if env_prefix.is_dir():
+            shutil.rmtree(env_prefix)
+
+        env_prefix.mkdir(parents=True, exist_ok=True)
+
+        # create the env at a temp dir
+        with tempfile.TemporaryDirectory() as temp_dir:
+
+            # install environment to temp dir
+            env_name = "env"
+            temp_env_dir = Path(temp_dir) / env_name
+            pkg_spec = f"{pkg_meta['name']}={pkg_meta['version']}={pkg_meta['build']}"
+            create_env(
+                pkg_spec, temp_env_dir, platform="emscripten-32", channels=channels
+            )
+
+            js_files = []
+            for pkg_meta in iterate_env_pkg_meta(temp_env_dir):
+
+                pkg_outname = pkg_meta["fn"][:-8]
+                pkg_name = pkg_meta["name"]
+
+                matchers = pkg_file_filter.get_matchers(pkg_name=pkg_name)
+
+                with tempfile.TemporaryDirectory() as temp_root_dir:
+
+                    temp_dirs = []
+                    target_dirs = []
+                    for i in range(len(matchers)):
+                        temp_dir = os.path.join(temp_root_dir, f"{i}")
+                        target_dir = os.path.join(temp_dir, env_name)
+                        os.makedirs(target_dir)
+                        temp_dirs.append(temp_dir)
+                        target_dirs.append(target_dir)
+
+                    has_any_files = split_filter_pkg(
+                        env_prefix=temp_env_dir,
+                        pkg_meta=pkg_meta,
+                        target_dirs=target_dirs,
+                        matchers=matchers,
+                    )
+
+                    js_files = []
+                    for i in range(len(matchers)):
+                        if has_any_files[i]:
+
+                            pkg_base_name = f"{pkg_outname}.{i}"
+
+                            emscripten_file_packager(
+                                outname=f"{pkg_base_name}",
+                                to_mount=env_name,
+                                mount_path=env_prefix,
+                                export_name=export_name,
+                                use_preload_plugins=True,
+                                no_node=export_name.startswith("globalThis"),
+                                lz4=True,
+                                cwd=str(temp_dirs[i]),
+                                download_emsdk=download_emsdk,
+                            )
+                            js_files.append(f"{pkg_base_name}.js")
+                            shutil.copy(
+                                os.path.join(
+                                    str(temp_dirs[i]), f"{pkg_base_name}.data"
+                                ),
+                                pack_outdir,
+                            )
+                            shutil.copy(
+                                os.path.join(str(temp_dirs[i]), f"{pkg_base_name}.js"),
+                                pack_outdir,
+                            )
+
+                            extend_js(os.path.join(pack_outdir, f"{pkg_base_name}.js"))
+
+                            js_files.append(f"{pkg_base_name}.js")
+
+                    with open(os.path.join(pack_outdir, f"{pkg_base_name}.js")) as f:
+                        json.dump(js_files, f)
+
+
 def pack_environment(
     env_prefix: Path,
     outname: str,
@@ -259,7 +365,6 @@ def pack_environment(
             target_dir=target_dir,
             pkg_file_filter=pkg_file_filter,
         )
-        shrink_conda_meta(env_prefix, target_dir=target_dir)
 
         emscripten_file_packager(
             outname=outname,
@@ -313,9 +418,46 @@ def pack_file(
         shutil.copy(os.path.join(temp_dir_str, f"{outname}.js"), os.getcwd())
 
 
-def create_env(pkg_name: str, prefix: str, platform: str):
+def pack_directory(
+    directory: Path,
+    mount_path: str,
+    outname: str,
+    export_name: str,
+    use_preload_plugins: bool = True,
+    silent: bool = False,
+):
+
+    with tempfile.TemporaryDirectory() as temp_dir:
+        temp_dir_str = str(temp_dir)
+
+        os.makedirs(os.path.join(temp_dir_str, "assets"), exist_ok=False)
+        assets_dir = os.path.join(temp_dir_str, "assets")
+        shutil.copytree(directory, assets_dir, dirs_exist_ok=True)
+
+        emscripten_file_packager(
+            outname=outname,
+            to_mount="assets/",
+            mount_path=mount_path,
+            export_name=export_name,
+            use_preload_plugins=use_preload_plugins,
+            no_node=export_name.startswith("globalThis"),
+            lz4=True,
+            cwd=temp_dir_str,
+            silent=silent,
+        )
+
+        shutil.copy(os.path.join(temp_dir_str, f"{outname}.data"), os.getcwd())
+        shutil.copy(os.path.join(temp_dir_str, f"{outname}.js"), os.getcwd())
+
+
+def create_env(pkg_name: str, prefix: str, platform: str, channels=None):
+
+    channel_args = ""
+    if channels is not None:
+        channel_args = " ".join([f" -c {c} " for c in channels])
+
     cmd = [
-        f"$MAMBA_EXE create --yes --prefix {prefix} --platform={platform} --no-deps {pkg_name}"
+        f"$MAMBA_EXE create --yes --prefix {prefix} {channel_args} --platform={platform} --no-deps {pkg_name} "
     ]
     ret = subprocess.run(cmd, shell=True, check=False, capture_output=True)
     if ret.returncode != 0:
